-- Useful file functions

function love.filesystem.rename(old, new)
  love.filesystem.write(new, love.filesystem.read(old))
  love.filesystem.remove(old)
end

function quitgame()
  love.mouse.setVisible(true)
  love.timer.sleep(0.5)
  love.event.quit()
end

function getordereddirectory(dir)
  local unordered = love.filesystem.getDirectoryItems(dir)
  local ordered = {}
  for k,v in pairs(unordered) do
    table.insert(ordered, v)
  end
  table.sort(ordered)
  return ordered
end

function copyfile(filetocopy, newlocation)
  local oldfile, data, newfile
  oldfile = love.filesystem.newFile(filetocopy, "r")
  if oldfile ~= nil then
    data = oldfile:read()
    oldfile:close()
    newfile = love.filesystem.newFile(newlocation, "w")  
    newfile:write(data)
    newfile:close()
  end
end

function readsinglelist(file)
  local tbl = {}
  if love.filesystem.getInfo(file, "file") then
    for line in love.filesystem.lines(file) do
      if string.len(line)>=1 then table.insert(tbl, line) end
    end
  end
  return tbl
end

function writesinglelist(filename, tbl)
  local file = love.filesystem.newFile(filename)
  local i
  if love.filesystem.getInfo(filename, "file") then love.filesystem.remove(filename) end
  file:open("w")
  if next(table)~=nil then
    for i = 1, #tbl, 1 do
      file:write(tbl[i].."\r\n")
    end
  end
  file:close()
end

function readsingletable(file)
  local p
  local tbl = {}
  if love.filesystem.getInfo(file, "file") then
    for line in love.filesystem.lines(file) do
      if string.find(line, "=") then
        p = string.find(line, "=")
        if p>1 and string.len(line)>p then
          if tonumber(string.sub(line, p+1)) then tbl[string.sub(line, 1, p-1)]=tonumber(string.sub(line, p+1))
          else tbl[string.sub(line, 1, p-1)]=string.sub(line, p+1) end
        end
      end
    end
  end
  return tbl
end

function writesingletable(filename, tbl)
  local file = love.filesystem.newFile(filename)
  if love.filesystem.getInfo(filename, "file") then love.filesystem.remove(filename) end
  file:open("w")
  if next(tbl)~=nil then
    for k, v in pairs(tbl) do
      file:write(tostring(k).."="..tostring(v).."\r\n")
    end
  end
  file:close()
end

function readmultitable(file)
  local i, p = 0, 0
  local tbl = {}
  if love.filesystem.getInfo(file, "file") then
    for line in love.filesystem.lines(file) do
      if string.find(line, "Name=") then
        i = i + 1
        tbl[i] = {}
        tbl[i].Name = string.sub(line, 6)
      elseif string.find(line, "=") then
        p = string.find(line, "=")
        if p>1 and string.len(line)>p then
          if tonumber(string.sub(line, p+1)) then tbl[i][string.sub(line, 1, p-1)]=tonumber(string.sub(line, p+1))
          else tbl[i][string.sub(line, 1, p-1)]=string.sub(line, p+1) end
        end
      else end
    end
  end
  return tbl
end

function writemultitable(filename, tbl)
  local file = love.filesystem.newFile(filename)
  local i
  if love.filesystem.getInfo(filename, "file") then love.filesystem.remove(filename) end
  file:open("w")
  if next(tbl)~=nil then
    for i = 1, #tbl, 1 do
      file:write("Name="..tbl[i].Name.."\r\n")
      for k, v in pairs(tbl[i]) do
        if tostring(k)~="Name" then file:write(tostring(k).."="..tostring(v).."\r\n") end
      end
    end
  end
  file:close()
end