-- "Wordplay"
-- An educational tool by Tim Sandford, 2020
-- Comments to timsandford42@hotmail.com

require("file42")
require("math42")
require("music42")
require("table42")

function love.run()   -- Main function
  gVERSION, gPATCH, gREVISION, gCODENAME = love.getVersion()
  if gVERSION >=  11 then  -- Colour fix for 11.0 onwards
    local oldSetColor = love.graphics.setColor
    love.graphics.setColor = function (r, g, b, a)     
      if type(r)=="table" then
        r, g, b, a = r[1] / 255, r[2] / 255, r[3] / 255, (r[4] or 255) / 255
      else
        r, g, b, a = r / 255, g / 255, b / 255, (a or 255) / 255
      end
      oldSetColor(r,g,b,a)
    end
  end
  math.randomseed(os.time())  -- True randomness
  love.load(love.arg.parseGameArguments(arg), arg) -- load variables etc.
  while true do  -- Main loop
    if love.event then -- Process events
      love.event.pump()
      for name, a,b,c,d,e,f in love.event.poll() do
        if name == "quit" then
          if not love.quit or not love.quit() then
            return a
          end
        end
        love.handlers[name](a,b,c,d,e,f)
      end
    end
    love.update() -- Update
    love.graphics.origin() -- Not sure, probably reset origin?
    love.graphics.clear(love.graphics.getBackgroundColor()) -- Clears the screen
    love.draw() -- Collect draw commands
    love.graphics.present() -- Present the graphics on the screen!
    love.timer.sleep(0.001) -- This is to lower the load on the CPU, but I could edit it :)	
  end
end

function love.load(arg)  -- Load everything in
  local i
  io.stdout:setvbuf("no")                                         -- Print messages immediately, allows tracing bugs
  if arg[#arg] == "-debug" then require("mobdebug").start() end   -- Zerobrane debugging
  gSCREEN_W, gSCREEN_H = love.graphics.getWidth(), love.graphics.getHeight() -- Get screensize
  gRATIOX, gRATIOY = gSCREEN_W/800, gSCREEN_H/600                 -- Scale graphics to screen
  gFADE, gFADESPEED = 255, 5                                      -- Fade in effect
  gTIMER = love.timer.getTime()                                   -- Use timer in love.update for speed of game
  gFLASH, gFLASH_z, gFLASH_min, gFLASH_max = 255, 10, 80, 255			-- Flashing images/text transparency
  gCOUNTER = 0                                                    -- Increases, fires events
  gTOUCH = false                                                  -- Becomes true after a touchpressed event

  love.mouse.setCursor(love.mouse.newCursor("gui/cursor.png", 21, 2))
  love.mouse.setVisible(true)
  
  fTINY = love.graphics.newFont("fonts/regular.ttf", 8)           -- tiny font
  fSMALL = love.graphics.newFont("fonts/regular.ttf", 12)         -- small font
  fDEFAULT = love.graphics.newFont("fonts/regular.ttf", 16)       -- default font
  fSUBHEADING = love.graphics.newFont("fonts/heading.ttf", 32)    -- sub heading font
  fHEADING = love.graphics.newFont("fonts/heading.ttf", 64)       -- heading font
  fTITLE = love.graphics.newFont("fonts/heading.ttf", 100)        -- title font
  fSUPER = love.graphics.newFont("fonts/heading.ttf", 200)        -- supersize font
    
  iTITLE = {}
  iICON = {}
  sSOUND = {}
  for i = 1, 99, 1 do
    if love.filesystem.getInfo("gui/title"..i..".png", "file") then iTITLE[i] = love.graphics.newImage("gui/title"..i..".png") end
    if love.filesystem.getInfo("gui/icon"..i..".png", "file") then iICON[i] = love.graphics.newImage("gui/icon"..i..".png") end
    if love.filesystem.getInfo("sounds/sound"..i..".ogg", "file") then sSOUND[i] = love.audio.newSource("sounds/sound"..i..".ogg", "static") end
  end
  iUNKNOWN = love.graphics.newImage("gui/unknown.png")
  
  if love.filesystem.getInfo("settings.txt", "file") then tSETTINGS = readsingletable("settings.txt")
  else tSETTINGS = readsingletable("data/defaultsettings.txt") end
  
  tMODES = readmultitable("data/modes.txt")
  tWORDS = readmultitable("data/words.txt")
  tGROUPS = readsinglelist("data/groups.txt")
  tCAPITALS = readmultitable("data/capitals.txt")
  
  tIMAGES = {}
  tSPEAK = {}
  if next(tWORDS)~=nil then
    for i = 1, #tWORDS, 1 do
      if love.filesystem.getInfo("images/"..tWORDS[i].Name..".jpg", "file") then tIMAGES[tWORDS[i].Name] = love.graphics.newImage("images/"..tWORDS[i].Name..".jpg") end
      if love.filesystem.getInfo("speech/"..tWORDS[i].Name..".ogg", "file") then tSPEAK[tWORDS[i].Name] = love.audio.newSource("speech/"..tWORDS[i].Name..".ogg", "static") end
    end
  end
  if next(tCAPITALS)~=nil then
    for i = 1, #tCAPITALS, 1 do
      if love.filesystem.getInfo("speech/"..tCAPITALS[i].Sound..".ogg", "file") then tSPEAK[tCAPITALS[i].Sound] = love.audio.newSource("speech/"..tCAPITALS[i].Sound..".ogg", "static") end
    end
  end
  
  musicload(tSETTINGS.MusicVol)
  love.mouse.setVisible(true)
  if tSETTINGS.Music=="Yes" then musicplay(1) end
  gGAMEMODE = "title"
  gFADE = 255
end

function startmenu()  -- Load the main menu
  gGAMEMODE = "menu"
  if tSETTINGS.Music=="Yes" then musicplay(1) end
  gFADE = 255
  gCOUNTER = 0
end

function startgame()  -- Start the game mode
  gINDEX, gGROUP, gSCORE, gTRIES, gTOTAL = 0, 0, 0, 0, 10
  if gPLAY == "Yes" then gTOTAL = 20 end
  if gGAMEMODE == "say" then
    gGROUP = 1
    choosewords(1)
  elseif gGAMEMODE == "spot" or gGAMEMODE == "rhyme" or gGAMEMODE == "hear" or gGAMEMODE == "spell" or gGAMEMODE == "fred" then
    choosewords()
  else end
end

function choosewords(k)  -- Choose the next word
  k = k or 0
  local list = {}
  local safety = 0
  local i, j
  gWORD, gLETTER, gCOUNTER = {}, {}, 0
  if gPLAY=="Yes" then
    if gGAMEMODE=="rhyme" then gGAMEMODE = "hear"
    elseif gGAMEMODE=="hear" then gGAMEMODE = "fred"
    elseif gGAMEMODE=="fred" then gGAMEMODE = "spell"
    elseif gGAMEMODE=="spell" then gGAMEMODE = "rhyme"
    else end
  end
  if gGAMEMODE == "say" then   -- Choose a word beginning with the right capital letter at random
    repeat
      safety = safety + 1
      if k~=0 then
        gINDEX = gINDEX + k
        if gINDEX < 1 then
          gINDEX, gGROUP = 26, gGROUP - 1
          if gGROUP < 1 then gGROUP = #tGROUPS end
        elseif gINDEX > 26 then
          gINDEX, gGROUP = 1, gGROUP + 1
          if gGROUP > #tGROUPS then gGROUP = 1 end
        else end
      end
      if next(tWORDS)~=nil and gINDEX~=nil and gINDEX>=1 and gINDEX<=26 and gGROUP~=nil and gGROUP>=1 and gGROUP<=#tGROUPS then
        for i = 1, #tWORDS, 1 do
          if string.upper(string.sub(tWORDS[i].Name, 1, 1))==tCAPITALS[gINDEX].Name and (tWORDS[i].Group==tGROUPS[gGROUP] or tGROUPS[gGROUP]=="Random") then 
            table.insert(list, i)
          end
        end
      end
    until next(list)~=nil or safety > 100
    if next(list)~=nil then
      table.insert(gWORD, deepcopy(tWORDS[list[math.random(#list)]]))
    end
  elseif gGAMEMODE == "spot" then     -- Choose four words at random, assigning one of them to be the correct one
    while #list < 4 do   -- Just check that there are four words available!
      list, gGROUP = {}, math.random(#tGROUPS)
      for i = 1, #tWORDS, 1 do
        if tWORDS[i].Group==tGROUPS[gGROUP] and tWORDS[i]["spot"]=="Yes" then table.insert(list, deepcopy(tWORDS[i])) end
      end
    end
    for i = 1, 4, 1 do    -- Now pick four words
      j = math.random(#list)
      table.insert(gWORD, deepcopy(list[j]))
      table.remove(list, j)
    end
    gINDEX = math.random(4)
  elseif gGAMEMODE == "rhyme" then    -- Choose one word at random that has at least one other rhyme
    repeat
      safety = safety + 1
      list, gINDEX = {}, math.random(#tWORDS)
      for i = 1, #tWORDS, 1 do
        if i~=gINDEX and tWORDS[i].rhyme=="Yes" and tWORDS[gINDEX].rhyme=="Yes" and tWORDS[i].RhymeGroup==tWORDS[gINDEX].RhymeGroup then
          table.insert(list, deepcopy(tWORDS[i]))
        end
      end
    until #list >= 1 or safety > 1000
    if #list >= 1 then                -- If there's at least one other rhyme...
      while #gWORD < 3 do             -- Get three words that don't rhyme
        i = math.random(#tWORDS)
        safety = true
        if next(gWORD)~=nil then
          for j = 1, #gWORD, 1 do
            if gWORD[j].Name == tWORDS[i].Name then safety = false end
          end
        end
        if tWORDS[i].rhyme=="Yes" and tWORDS[i].RhymeGroup~=tWORDS[gINDEX].RhymeGroup and safety==true then
          table.insert(gWORD, deepcopy(tWORDS[i]))
        end
      end
      i, j = math.random(#list), math.random(4)   -- Insert rhyming word in random spot
      table.insert(gWORD, j, deepcopy(list[i]))
    end
  elseif gGAMEMODE == "hear" then   -- Choose (up to) four words that rhyme and assign one to be correct
    for i = 1, #tWORDS, 1 do        -- Add all the possible words for this gamemode
      if tWORDS[i].hear=="Yes" then table.insert(list, deepcopy(tWORDS[i])) end
    end
    if next(list)~=nil then
      j = math.random(#list)        -- Choose a random word from the possibles to rhyme with
      for i = 1, #list, 1 do        -- Add all the words that rhyme with it (could be more than four)
        if list[i].RhymeGroup==list[j].RhymeGroup then table.insert(gWORD, deepcopy(list[i])) end
      end
      while #gWORD > 4 do           -- Cut the table down to size if it's greater than four
        table.remove(gWORD, math.random(#gWORD))
      end
      gINDEX = math.random(#gWORD)
    end
  elseif gGAMEMODE == "spell" or gGAMEMODE == "fred" then  -- Choose a word to spell and randomise the letters/phonemes
    gINDEX = 0
    for i = 1, #tWORDS, 1 do        -- Add all the spelling options to a list
      if tWORDS[i][gGAMEMODE]=="Yes" then table.insert(list, deepcopy(tWORDS[i])) end
    end
    if next(list)~=nil then
      i = math.random(#list)        -- Add one word to spell at random
      table.insert(gWORD, deepcopy(list[i]))
      gLETTER = getletters(gWORD[1])    -- Define the letter pieces
      if #gLETTER >= 1 then             -- Space them out
        j = 700 / #gLETTER
        for i = 1, #gLETTER, 1 do
          gLETTER[i].X = (((i-1)*j)+(i*j))/2
          gLETTER[i].Y = 470
        end
        if #gLETTER >= 2 then          -- Shuffle the letters
          for i = 1, #gLETTER-1, 1 do
            j = math.random(#gLETTER-i)+i
            gLETTER[i].X, gLETTER[j].X = gLETTER[j].X, gLETTER[i].X
          end
        end
      end
    end
  else end
end

function love.update()  -- Runs every frame
  updategeneral()
  updatecounter()
  musicupdate()
  collectgarbage()
  if love.timer.getTime()-gTIMER<=(1/tSETTINGS.FPS) then love.timer.sleep((1/tSETTINGS.FPS)-(love.timer.getTime()-gTIMER)) end  -- frame rate
  gTIMER = love.timer.getTime()  -- new timer
end

function updategeneral()  -- Updates general variables, fade and flash
  if gFADE > 0 then  -- fade in graphics
    gFADE = gFADE - gFADESPEED
    if gFADE < 0 then gFADE = 0 end
  end
  gFLASH = gFLASH + gFLASH_z  -- flash counter
  if gFLASH > 255 then  -- this affects everything that flashes (same speed)
    gFLASH = gFLASH_max
    gFLASH_z = 0 - gFLASH_z
  elseif gFLASH < gFLASH_min then
    gFLASH = gFLASH_min
    gFLASH_z = 0 - gFLASH_z
  else end
end

function updatecounter()  -- Update the counter
  local mx, my = love.mouse.getX()/gRATIOX, love.mouse.getY()/gRATIOY
  local changed = false
  local i, j, k
  gCOUNTER = math.min(gCOUNTER+1, 3600)
  if gGAMEMODE == "title" then
    if gCOUNTER > 1800 then startmenu() end
  elseif gGAMEMODE == "menu" then
    if gCOUNTER > 1800 then 
      gPLAY = "No"
      gGAMEMODE = "say"
      gFADE = 255
      startgame()
    end
  elseif gGAMEMODE == "say" then
    if gCOUNTER==30 and gINDEX~=nil and gINDEX>=1 and gINDEX<=26 and next(tCAPITALS)~=nil and next(tSPEAK)~=nil and tSPEAK[tCAPITALS[gINDEX].Sound]~=nil then love.audio.play(tSPEAK[tCAPITALS[gINDEX].Sound])
    elseif gCOUNTER==90 and next(gWORD)~=nil and next(tSPEAK)~=nil and tSPEAK[gWORD[1].Name]~=nil then love.audio.play(tSPEAK[gWORD[1].Name])
    elseif gCOUNTER==150 and next(gWORD)~=nil and gWORD[1].Sound~=nil then playsound(gWORD[1].Sound, "Yes")
    elseif gCOUNTER>1800 then 
      choosewords(1)
    else end
  elseif gGAMEMODE == "spot" or gGAMEMODE == "hear" then
    if gCOUNTER==10 then playsound(27, "Yes")
    elseif gCOUNTER==90 and next(gWORD)~=nil and next(tSPEAK)~=nil and tSPEAK[gWORD[gINDEX].Name]~=nil then love.audio.play(tSPEAK[gWORD[gINDEX].Name])
    elseif gCOUNTER==300 then gCOUNTER = 200
    elseif gCOUNTER==500 then
      if gTRIES == gTOTAL then 
        if gSCORE==gTOTAL then playsound(30, "Yes") end
        gGAMEMODE = "score"
        gCOUNTER = 0
      else choosewords() end
    elseif gCOUNTER>560 and gWORD[gINDEX].Sound~=nil and not sSOUND[gWORD[gINDEX].Sound]:isPlaying() then gCOUNTER = 400
    else end
  elseif gGAMEMODE == "rhyme" then
    if gCOUNTER==10 then playsound(31, "Yes")
    elseif gCOUNTER==140 and gINDEX~=nil and tSPEAK[tWORDS[gINDEX].Name]~=nil then love.audio.play(tSPEAK[tWORDS[gINDEX].Name])
    elseif gCOUNTER==300 then gCOUNTER = 200
    elseif gCOUNTER==500 then
      if gTRIES == gTOTAL then
        if gSCORE==gTOTAL then playsound(30, "Yes") end
        gGAMEMODE = "score"
        gCOUNTER = 0
      else choosewords() end
    elseif gCOUNTER==750 then gCOUNTER = 400
    else end
  elseif gGAMEMODE == "fred" or gGAMEMODE=="spell" then
    if gCOUNTER==10 then 
      if gGAMEMODE=="fred" then playsound(33, "Yes")
      else playsound(32, "Yes") end
    elseif gCOUNTER == 130 and next(gWORD)~=nil and next(tSPEAK)~=nil and tSPEAK[gWORD[1].Name]~=nil then love.audio.play(tSPEAK[gWORD[1].Name])
    elseif gCOUNTER==300 then gCOUNTER = 200
    elseif gCOUNTER>=400 and gCOUNTER<=540 and next(gLETTER)~=nil then
      k = 600 / (#gLETTER+1)
      for i = 1, #gLETTER, 1 do
        for j = 1, #gLETTER, 1 do
          if gLETTER[i].X == 50+(j*k) and gLETTER[i].Y == 350 and gCOUNTER==400+math.floor((j/#gLETTER)*140) then
            if tSPEAK[gLETTER[i].Phonic]~=nil then 
              love.audio.stop(tSPEAK[gLETTER[i].Phonic])
              love.audio.play(tSPEAK[gLETTER[i].Phonic]) 
            end
          end
        end
      end
    elseif gCOUNTER==600 then
      if tSPEAK[gWORD[1].Name]~=nil then love.audio.play(tSPEAK[gWORD[1].Name]) end
    elseif gCOUNTER==700 then
      if gWORD[1].Sound~=nil then 
        playsound(gWORD[1].Sound)
        gCOUNTER=750
      end
    elseif gCOUNTER==720 then
      if gTRIES == gTOTAL then 
        if gSCORE==gTOTAL then playsound(30, "Yes") end
        gGAMEMODE = "score"
        gCOUNTER = 0
      else 
        changed = true
        choosewords() 
      end
    elseif gCOUNTER>760 and not sSOUND[gWORD[1].Sound]:isPlaying() then gCOUNTER=710
    else end  
    if gINDEX ~= 0 and changed==false then gLETTER[gINDEX].X, gLETTER[gINDEX].Y = mx, my end
  elseif gGAMEMODE == "score" then
    if gCOUNTER==300 then startmenu() end
  else end
end

function love.draw()  -- Draw everything in, based on gamemode
  love.graphics.scale(gRATIOX, gRATIOY)
  if _G["draw"..gGAMEMODE]~=nil then _G["draw"..gGAMEMODE]() end
  if gFADE > 0 then 
    love.graphics.setColor(0, 0, 0, gFADE)
    love.graphics.rectangle("fill", 0, 0, 800, 600)
  else end
end

function drawbackground()  -- Draw the background
  love.graphics.setColor(225, 225, 225, 255)
  love.graphics.rectangle("fill", 0, 0, 800, 600)
end

function drawtitle()  -- Draw the title page
  drawbackground()
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.draw(iTITLE[1], 200, 195)
end

function drawmenu()  -- Draw the main menu
  local mx, my = love.mouse.getX()/gRATIOX, love.mouse.getY()/gRATIOY
  local i
  drawbackground()
  love.graphics.setFont(fSUBHEADING)
  for i = 2, #iTITLE-1, 1 do
    if mx>100 and mx<300 and my>(70*i)-126 and my<(70*i)-74 and gTOUCH==false then 
      love.graphics.setColor(255, 255, 255, 255)
      love.graphics.draw(iTITLE[i], 200, (70*i)-100, 0, 0.66, 0.66, 200, 53)
      love.graphics.setColor(235, 70, 145, 255)
      love.graphics.printf(tMODES[i].Desc, 350, (70*i)-136, 400, "left")
    else 
      love.graphics.setColor(200, 200, 200, 255)
      love.graphics.draw(iTITLE[i], 200, (70*i)-100, 0, 0.5, 0.5, 200, 53)
    end
  end
  if mx>550 and mx<750 and my>490 and my<595 and gTOUCH==false then
    love.graphics.setColor(255, 255, 255, 255)
    love.graphics.draw(iTITLE[#iTITLE], 650, 542.5, 0, 0.66, 0.66, 200, 53)
  else
    love.graphics.setColor(200, 200, 200, 255)
    love.graphics.draw(iTITLE[#iTITLE], 650, 542.5, 0, 0.5, 0.5, 200, 53)
  end
  love.graphics.setColor(235, 70, 145, 255)
  if tSETTINGS.Music == "Yes" then love.graphics.draw(iICON[1], 15, 530)
  else love.graphics.draw(iICON[2], 15, 530) end
  love.graphics.printf("music by bensound.com", 75, 540, 400, "left")
end

function drawcategories()
  local l, h = getgroupdimensions()
  local k, p, s = 0, 0, 0
  local i, j
  drawbackground()
  if next(tGROUPS)~=nil and next(tWORDS)~=nil then
    for j = 1, h, 1 do
      for i = 1, l, 1 do
        k = k + 1
        s, p = 0, 0
        repeat
          p, s = p + 1, s + 1
          if p > #tWORDS then p = 1 end
        until tWORDS[p].Group==tGROUPS[k] or s > #tWORDS or tGROUPS[k]=="Random"
        if tGROUPS[k]=="Random" then 
          drawpicture(iICON[8], (i*800/l)-(400/l), (j*600/h)-(300/h), 800/l, 600/h)
        elseif s<=#tWORDS and tIMAGES[tWORDS[p].Name]~=nil then 
          drawpicture(tIMAGES[tWORDS[p].Name], (i*800/l)-(400/l), (j*600/h)-(300/h), 800/l, 600/h)
        else end
        if tGROUPS[k]~=nil then drawword(tGROUPS[k], (i*800/l)-(400/l), (j*600/h)-(300/h), 800/l) end
      end
    end
  end
  drawcross()
end

function drawsay()
  drawbackground()
  if gINDEX~=nil and gINDEX>=1 and gINDEX<=26 and gCOUNTER>=30 then drawletter(tCAPITALS[gINDEX].Name, 230, 200, 300, 300, fSUPER) end
  if next(gWORD)~=nil then
    if tIMAGES[gWORD[1].Name]~=nil and gCOUNTER>=90 then drawpicture(tIMAGES[gWORD[1].Name], 570, 200, 300, 300) end
    if sSOUND[gWORD[1].Sound]~=nil and gCOUNTER>=90 and tSETTINGS.Sound=="Yes" then drawsoundicon(570, 200, 300, 300) end
    drawword(gWORD[1].Name, 400, 370, 800)
  end
  drawarrows()
  drawcross()
  drawrepeat()
end

function drawspot()
  local x, y = {275, 525, 275, 525}, {175, 175, 425, 425}
  local i
  drawbackground()
  if next(gWORD)~=nil then
    for i = 1, 4, 1 do
      if #gWORD>=i and gWORD[i].Name~=nil and tIMAGES[gWORD[i].Name]~=nil and gCOUNTER>=90+(i*10) then 
        drawpicture(tIMAGES[gWORD[i].Name], x[i], y[i], 250, 250)
        drawword(gWORD[i].Name, x[i], y[i]+85, 250, fSUBHEADING)
      end
    end
  end
  drawsidebar()
  drawcross()
  drawrepeat()
end

function drawrhyme()
  local mx, my = love.mouse.getX()/gRATIOX, love.mouse.getY()/gRATIOY
  local x, y = {270, 430, 270, 430}, {320, 320, 480, 480}
  local i
  drawbackground()
  if gINDEX ~= 0 and gCOUNTER>=140 and tIMAGES[tWORDS[gINDEX].Name]~=nil then
    drawpicture(tIMAGES[tWORDS[gINDEX].Name], 250, 100, 160, 160)
    drawword(tWORDS[gINDEX].Name, 450, 50, 160)
  end
  if next(gWORD)~=nil then
    for i = 1, 4, 1 do
      if gWORD[i].Name~=nil and tIMAGES[gWORD[i].Name]~=nil and gCOUNTER>=150 then
        drawpicture(tIMAGES[gWORD[i].Name], x[i], y[i], 160, 160)
        if mx>=x[i]-80 and mx<=x[i]+80 and my>=y[i]-80 and my<=y[i]+80 and gTOUCH==false then
          love.graphics.setColor(255, 255, 255, 180)
          love.graphics.rectangle("fill", x[i]-80, y[i]-80, 160, 160)
          drawword(gWORD[i].Name, x[i], y[i]-20, 250, fSUBHEADING)
        end
      end
    end
  end
  drawsidebar()
  drawcross()
  drawrepeat()
end

function drawhear()
  drawspot()
end

function drawfred()
  drawbackground()
  drawboxes()
  if tIMAGES[gWORD[1].Name]~=nil then drawpicture(tIMAGES[gWORD[1].Name], 350, 150, 200, 200) end
  drawsidebar()
  drawletters()
  drawcross()
  drawrepeat()
end

function drawspell()
  drawfred()
end

function drawletters()
  local i, j ,k
  if next(gLETTER)~=nil then
    k = 600 / (#gLETTER+1)
    for i = 1, #gLETTER, 1 do
      if (gGAMEMODE=="fred" or gGAMEMODE=="spell") and gCOUNTER>=400 then 
        for j = 1, #gLETTER, 1 do
          if gLETTER[i].X==50+(j*k) and gLETTER[i].Y==350 then 
            if gCOUNTER>400+math.floor((j/#gLETTER)*140) and gCOUNTER<420+math.floor((j/#gLETTER)*140) then
              drawletter(gLETTER[i].Letter, gLETTER[i].X, gLETTER[i].Y-25, 125, 125, fHEADING)
            else drawletter(gLETTER[i].Letter, gLETTER[i].X, gLETTER[i].Y, 100, 100, fHEADING) end
          end
        end
      else drawletter(gLETTER[i].Letter, gLETTER[i].X, gLETTER[i].Y, 100, 100, fHEADING) end
    end
  end
end

function drawboxes()
  local i, j
  if next(gLETTER)~=nil then
    j = 600 / (#gLETTER+1)
    for i = 1, #gLETTER, 1 do
      love.graphics.setColor(0, 0, 0, 150)
      love.graphics.rectangle("fill", (i*j)+3, 303, 100, 100)
      love.graphics.setColor(255, 255, 255, 255)
      love.graphics.rectangle("fill", (i*j), 300, 100, 100)
    end
  end
end

function drawsidebar()
  local i
  love.graphics.setColor(235, 70, 145, 50)
  love.graphics.rectangle("fill", 700, 0, 100, 600)
  love.graphics.setColor(235, 70, 145, 255)
  love.graphics.rectangle("fill", 700, 600-(gSCORE*(600/gTOTAL)), 100, gSCORE*(600/gTOTAL))
  love.graphics.setColor(255, 255, 255, 255)
  for i = 0, gTOTAL, 1 do
    love.graphics.line(700, (i*(600/gTOTAL)), 740, (i*(600/gTOTAL)))
  end
  love.graphics.setFont(fSUBHEADING)
  love.graphics.printf(gSCORE.."/"..gTRIES, 700, math.max(572-(gSCORE*(600/gTOTAL)),10), 100, "center")
end

function drawscore()
  drawbackground()
  love.graphics.setFont(fHEADING)  
  love.graphics.setColor(0, 0, 0, 150)
  love.graphics.printf("Your score: "..gSCORE.." / "..gTRIES, 3, 253, 800, "center")
  love.graphics.setColor(235, 70, 145, 255)
  love.graphics.printf("Your score: "..gSCORE.." / "..gTRIES, 0, 250, 800, "center")
  drawcross()
end

function drawletter(l, x, y, w, h, f)
  local s = h*0.8/f:getHeight()
  love.graphics.setFont(f)
  love.graphics.setColor(0, 0, 0, 150)
  love.graphics.rectangle("fill", x-(w/2)+3, y-(h/2)+3, w, h)
  love.graphics.setColor(235, 70, 145, 255)
  love.graphics.rectangle("fill", x-(w/2), y-(h/2), w, h)
  love.graphics.setColor(0, 0, 0, 150)
  love.graphics.printf(l, x+3, y-(h*0.4)+3, w, "center", 0, s, s, w/2, 0)
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.printf(l, x, y-(h*0.4), w, "center", 0, s, s, w/2, 0)
  if string.len(l)>=2 then love.graphics.line(x-(w*0.4), y+(h*0.4), x+(w*0.4), y+(h*0.4)) end
end

function drawpicture(image, x, y, w, h)
  love.graphics.setColor(0, 0, 0, 150)
  love.graphics.rectangle("fill", x-(w/2)+3, y-(h/2)+3, w, h)
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.draw(image, x, y, 0, w/image:getWidth(), h/image:getHeight(), image:getWidth()/2, image:getHeight()/2)
end

function drawsoundicon(x, y, w, h)
  love.graphics.setColor(0, 0, 0, 150)
  love.graphics.draw(iICON[6], x+(w*0.25)+3, y-(h*0.25)+3, 0, (w/2)/iICON[6]:getWidth(), (h/2)/iICON[6]:getHeight(), iICON[6]:getWidth()/2, iICON[6]:getHeight()/2)
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.draw(iICON[6], x+(w*0.25), y-(h*0.25), 0, (w/2)/iICON[6]:getWidth(), (h/2)/iICON[6]:getHeight(), iICON[6]:getWidth()/2, iICON[6]:getHeight()/2)
end

function drawword(word, x, y, w, fixed)
  fixed = fixed or nil
  local fonts = {fSUPER, fTITLE, fHEADING, fSUBHEADING} 
  local f = 1
  width, wrappedtext = fonts[f]:getWrap(word, w)
  while #wrappedtext>1 and f<#fonts do
    f = f + 1
    width, wrappedtext = fonts[f]:getWrap(word, w)
  end
  if fixed~=nil then love.graphics.setFont(fixed) else love.graphics.setFont(fonts[f]) end
  love.graphics.setColor(0, 0, 0, 150)
  love.graphics.printf(word, x-(w/2)+2, y+2, w, "center")
  love.graphics.setColor(235, 70, 145, 255)
  love.graphics.printf(word, x-(w/2), y, w, "center")
end

function drawarrows()
  local mx, my = love.mouse.getX()/gRATIOX, love.mouse.getY()/gRATIOY
  love.graphics.setColor(0, 0, 0, 150)
  love.graphics.draw(iICON[3], 7, 252)
  love.graphics.draw(iICON[4], 729, 252)
  if mx>=5 and mx<=73 and my>=250 and my<=318 and gTOUCH==false then love.graphics.setColor(235, 70, 145, 255)
  else love.graphics.setColor(255, 255, 255, 255) end
  love.graphics.draw(iICON[3], 5, 250)
  if mx>=727 and mx<=795 and my>=250 and my<=318 and gTOUCH==false then love.graphics.setColor(235, 70, 145, 255)
  else love.graphics.setColor(255, 255, 255, 255) end
  love.graphics.draw(iICON[4], 727, 250)
end

function drawcross()
  local mx, my = love.mouse.getX()/gRATIOX, love.mouse.getY()/gRATIOY
  love.graphics.setColor(0, 0, 0, 150)
  love.graphics.draw(iICON[5], 7, 7)
  if mx>=5 and mx<=69 and my>=5 and my<=69 and gTOUCH==false then love.graphics.setColor(235, 70, 145, 255)
  else love.graphics.setColor(255, 255, 255, 255) end
  love.graphics.draw(iICON[5], 5, 5)
end

function drawrepeat()
  local mx, my = love.mouse.getX()/gRATIOX, love.mouse.getY()/gRATIOY
  love.graphics.setColor(0, 0, 0, 150)
  love.graphics.draw(iICON[7], 7, 533)
  if mx>=5  and mx<=69 and my>=531 and my<=595 and gTOUCH==false then love.graphics.setColor(235, 70, 145, 255)
  else love.graphics.setColor(255, 255, 255, 255) end
  love.graphics.draw(iICON[7], 5, 531)
end

function love.touchpressed(id, x, y, dx, dy, pressure )
  gTOUCH = true
end

function love.mousepressed(x, y, button, istouch)  -- Record any click, based on gamemode
  x,y = x/gRATIOX, y/gRATIOY
  if _G["click"..gGAMEMODE]~=nil then
    _G["click"..gGAMEMODE](x,y)
    playsound(1)
  end
end

function clicktitle(x,y)
  startmenu()
end

function clickmenu(x,y)
  local i
  for i = 2, #iTITLE-1, 1 do
    if x>100 and x<300 and y>(70*i)-126 and y<(70*i)-74 then
      gFADE, gPLAY = 255, "No"
      gGAMEMODE = tMODES[i].Mode
      if gGAMEMODE=="say" then gGAMEMODE="categories" end
      if gGAMEMODE=="play" then 
        gPLAY = "Yes"
        gGAMEMODE = "spell"
      end
      startgame()
    end
  end
  if x>15 and x<65 and y>540 and y<590 then
    if tSETTINGS.Music=="Yes" then 
      tSETTINGS.Music="No"
      musicstop()
    else 
      tSETTINGS.Music="Yes" 
      musicplay(1)
    end
  end
  if x>550 and x<750 and y>490 and y<595 then quitgame() end
end

function clickcategories(x,y)
  local l, h = getgroupdimensions()
  local k, s = 0, 0
  local i, j
  if x>=5 and x<=69 and y>=5 and y<=69 then         -- Clicked on exit
    startmenu()
  else
    for j = 1, h, 1 do
      for i = 1, l, 1 do
        k = k + 1
        if x>(i-1)*(800/l) and x<i*800/l and y>(j-1)*(600/h) and y<j*600/h then s = k end
      end
    end
    if s~=0 and s<=#tGROUPS then
      gGROUP = s
      gGAMEMODE = "say"
      choosewords(1)
    end
  end
end

function clicksay(x,y)
  if x>=5 and x<=73 and y>=250 and y<=318 then          -- Clicked left arrow
    choosewords(-1)
  elseif x>=80 and x<=380 and y>=50 and y<=350 then     -- Clicked on letter
    if gINDEX~=nil and gINDEX>=1 and gINDEX<=26 and next(tCAPITALS)~=nil and next(tSPEAK)~=nil and tSPEAK[tCAPITALS[gINDEX].Sound]~=nil then 
      love.audio.stop(tSPEAK[tCAPITALS[gINDEX].Sound])
      love.audio.play(tSPEAK[tCAPITALS[gINDEX].Sound]) 
    end
  elseif x>=420 and x<=720 and y>=50 and y<=350 then    -- Clicked on image or sound
    if x>=570 and x<=720 and y>=50 and y<=200 and next(gWORD)~=nil and gWORD[1].Sound~=nil and tSETTINGS.Sound=="Yes" then playsound(gWORD[1].Sound, "Yes")
    elseif next(gWORD)~=nil and next(tSPEAK)~=nil and tSPEAK[gWORD[1].Name]~=nil then love.audio.play(tSPEAK[gWORD[1].Name])
    else end
  elseif x>=5 and x<=69 and y>=5 and y<=69 then         -- Clicked on exit
    startmenu()
  elseif x>=5 and x<=69 and y>=531 and y<=595 then      -- Clicked on repeat
    gCOUNTER = 0
  else 
    choosewords(1)
  end
end

function clickspot(x,y)
  local clicked = 0
  if x>150 and x<400 and y>50 and y<300 then clicked = 1
  elseif x>400 and x<650 and y>50 and y<300 then clicked = 2
  elseif x>150 and x<400 and y>300 and y<550 then clicked = 3
  elseif x>400 and x<650 and y>300 and y<550 then clicked = 4
  else end
  if clicked ~= 0 and clicked<=#gWORD and gCOUNTER>90 and gCOUNTER<400 then
    gCOUNTER = 400
    if clicked == gINDEX then
      if gWORD[gINDEX].Sound~=nil then 
        playsound(gWORD[gINDEX].Sound) 
        gCOUNTER = 550
      end
      gSCORE = gSCORE + 1
      playsound(28, "Yes")
    else playsound(29, "Yes") end
    gTRIES = gTRIES + 1
  elseif x>=5 and x<=69 and y>=5 and y<=69 then         -- Clicked on exit
    startmenu()
  elseif x>=5 and x<=69 and y>=531 and y<=595 and gCOUNTER<400 then      -- Clicked on repeat
    gCOUNTER = 0
  else end
end

function clickrhyme(x,y)
  local clicked = 0
  if x>190 and x<350 and y>240 and y<380 then clicked = 1
  elseif x>350 and x<510 and y>240 and y<380 then clicked = 2
  elseif x>190 and x<350 and y>400 and y<560 then clicked = 3
  elseif x>350 and x<510 and y>400 and y<560 then clicked = 4
  else end
  if clicked ~= 0 and gCOUNTER>150 and gCOUNTER<350 then
    gCOUNTER = 400
    if gWORD[clicked].RhymeGroup == tWORDS[gINDEX].RhymeGroup then
      if gWORD[clicked].Sound~=nil then 
        playsound(gWORD[clicked].Sound)
        gCOUNTER = 550
      end
      gSCORE = gSCORE + 1
      playsound(28, "Yes")
    else playsound(29, "Yes") end
    gTRIES = gTRIES + 1
  elseif x>=5 and x<=69 and y>=5 and y<=69 then         -- Clicked on exit
    startmenu()
  elseif x>=5 and x<=69 and y>=531 and y<=595 and gCOUNTER<350 then      -- Clicked on repeat
    gCOUNTER = 0
  else end
end

function clickhear(x,y)
  clickspot(x,y)
end

function clickfred(x,y)
  local clicked = 0
  local i, letter
  if next(gLETTER)~=nil then
    for i = 1, #gLETTER, 1 do
      if x>gLETTER[i].X-50 and x<gLETTER[i].X+50 and y>gLETTER[i].Y-50 and y<gLETTER[i].Y+50 then
        clicked = i
      end
    end
  end
  if x>=5 and x<=69 and y>=5 and y<=69 then         -- Clicked on exit
    startmenu()
  elseif x>=5 and x<=69 and y>=531 and y<=595 and gCOUNTER<400 then      -- Clicked on repeat
    gCOUNTER = 0
  elseif clicked ~= 0 and gCOUNTER<400 then 
    if gLETTER[clicked].Phonic~=nil and next(tSPEAK)~=nil and tSPEAK[gLETTER[clicked].Phonic]~=nil then
      love.audio.stop(tSPEAK[gLETTER[clicked].Phonic])
      love.audio.play(tSPEAK[gLETTER[clicked].Phonic])
    end
    table.insert(gLETTER, deepcopy(gLETTER[clicked]))
    table.remove(gLETTER, clicked)
    gINDEX = #gLETTER
  else end  
end

function clickspell(x,y)
  local clicked = 0
  local i, letter
  if next(gLETTER)~=nil then
    for i = 1, #gLETTER, 1 do
      if x>gLETTER[i].X-50 and x<gLETTER[i].X+50 and y>gLETTER[i].Y-50 and y<gLETTER[i].Y+50 then
        clicked = i
      end
    end
  end
  if x>=5 and x<=69 and y>=5 and y<=69 then         -- Clicked on exit
    startmenu()
  elseif x>=5 and x<=69 and y>=531 and y<=595 and gCOUNTER<400 then      -- Clicked on repeat
    gCOUNTER = 0
  elseif clicked ~= 0 and gCOUNTER<400 then 
    if gLETTER[clicked].Phonic~=nil and next(tSPEAK)~=nil and tSPEAK[gLETTER[clicked].Phonic]~=nil then
      love.audio.stop(tSPEAK[gLETTER[clicked].Phonic])
      love.audio.play(tSPEAK[gLETTER[clicked].Phonic])
    end
    table.insert(gLETTER, deepcopy(gLETTER[clicked]))
    table.remove(gLETTER, clicked)
    gINDEX = #gLETTER
  else end  
end

function clickscore(x,y)
  if x>=5 and x<=69 and y>=5 and y<=69 then         -- Clicked on exit
    startmenu()
  end
end

function love.mousereleased(x, y, button, istouch, presses)
  x,y = x/gRATIOX, y/gRATIOY
  if _G["release"..gGAMEMODE]~=nil then
    _G["release"..gGAMEMODE](x,y)
  end
end

function releasespell(x,y)
  if gCOUNTER<400 then snapboxes() end
  gINDEX = 0
end

function releasefred(x,y)
  if gCOUNTER<400 then snapboxes() end
  gINDEX = 0
end

function snapboxes()
  local filled, correct, once = 0, 0, false
  local i, j, k
  if next(gLETTER)~=nil then
    k = 600 / (#gLETTER+1)
    for i = 1, #gLETTER, 1 do
      once = false
      for j = 1, #gLETTER, 1 do
        if gLETTER[j].X>=(i*k) and gLETTER[j].X<=(i*k)+100 and gLETTER[j].Y>=300 and gLETTER[j].Y<=400 then
          once = true
          gLETTER[j].X, gLETTER[j].Y = 50+(i*k), 350
          if gGAMEMODE=="spell" and string.sub(gWORD[1].Name, i, i)==gLETTER[j].Letter then correct = correct + 1
          elseif gGAMEMODE=="fred" and gWORD[1]["Fred"..i]==gLETTER[j].Letter then correct = correct + 1
          else end
        end
      end
      if once == true then filled = filled + 1 end
    end
    if filled == #gLETTER then    -- All the boxes are full
      if correct == filled then
        gSCORE = gSCORE + 1
        playsound(28, "Yes")
      else playsound(29, "Yes") end
      gTRIES = gTRIES + 1
      gCOUNTER = 400
    end
  end
end

function love.keypressed(key)  -- Record any keypress, based on gamemode
  if _G["key"..gGAMEMODE]~=nil then 
    _G["key"..gGAMEMODE](key)
  end
end  

function keysay(key)
  if key=="escape" then
    startmenu()
  elseif key=="left" then
    choosewords(-1)
  elseif key=="right" then
    choosewords(1)
  elseif key=="return" then
    gCOUNTER = 0
  else end
end

function keyspot(key)
  if key=="escape" then
    startmenu()
  end
end

function keyrhyme(key)
  if key=="escape" then
    startmenu()
  end
end

function keyhear(key)
  if key=="escape" then
    startmenu()
  end
end

function keyspell(key)
  if key=="escape" then
    startmenu()
  end
end

function getgroupdimensions()
  local l, h, s = 0, 0, 0
  while l*h < #tGROUPS do
    if s==0 then
      l = l + 1
      s = 1
    else
      h = h + 1
      s = 0
    end
  end
  return l, h
end

function getletters(word)
  local letters, letter = {}, {}
  local i
  if string.len(word.Name)>= 1 then
    if gGAMEMODE == "spell" then
      for i = 1, string.len(word.Name), 1 do
        letter[i]={}
        letter[i].Letter = string.sub(word.Name, i, i)
        letter[i].Phonic = "capital "..letter[i].Letter
        table.insert(letters, deepcopy(letter[i]))
      end
    elseif gGAMEMODE == "fred" then
      for i = 1, 7, 1 do
        if word["Phonic"..i] and word["Fred"..i] then 
          letter[i]={}
          letter[i].Letter = word["Fred"..i]
          letter[i].Phonic = word["Phonic"..i]
          if love.filesystem.getInfo("speech/"..letter[i].Phonic..".ogg", "file") then
            tSPEAK[letter[i].Phonic] = love.audio.newSource("speech/"..letter[i].Phonic..".ogg", "static")
          end
          table.insert(letters, deepcopy(letter[i]))
        end
      end
    else end
  end
  return letters
end

function playsound(i, interrupt)  -- Play sound
  interrupt = interrupt or "No"
  if sSOUND[i]~=nil and tSETTINGS.Sound=="Yes" then 
    if interrupt == "Yes" then 
      sSOUND[i]:stop()
      love.audio.play(sSOUND[i])
    else
      if not sSOUND[i]:isPlaying() then love.audio.play(sSOUND[i]) end
    end
  end
end